%% Code accompanying "Uncertainty Quantification and Global Sensitivity Analysis for Economic Models"
%% by Daniel Harenberg, Stefano Marelli, Bruno Sudret, and Viktor Winschel

function ssD = M_D_steady_state(par)
xcell = num2cell(par);                        % create compound argument for steady state function
[k,a,c,l,i,y] = M_steady_state(xcell{:});     % evaluate at parameters  
ssC = [k,a,c,l,i,y];
ssD = zeros(8,6);                             % preallocate for steady state at changed parameters
d = eps/2;                                    % small difference
for p=1:8
    dP      = par;                           % change in parameters
    dP(p)   = par(p) + d;                     
    xcell   = num2cell(dP);                   % eval steady state
    [k,a,c,l,i,y] = M_steady_state(xcell{:});
    ssD(p,:) = ([k,a,c,l,i,y] - ssC ) / d;    % finite difference derivative
end
ssD = ssD .* repmat(par',1,6) ./ repmat(ssC,8,1);  % elasticity
