%% Code accompanying "Uncertainty Quantification and Global Sensitivity Analysis for Economic Models"
%% by Daniel Harenberg, Stefano Marelli, Bruno Sudret, and Viktor Winschel

% MFDP07 Function file for optimal growth model
function [out1,out2,out3,out4] = M_RBC(flag,s,x,z,e,sn,xn,beta,tau,theta,alpha,delta,phi,rho)

n   = size(s,1);

switch flag

case 'b'; % BOUND FUNCTION
  % xl
  out1  = [(zeros(n,2) + 1^(-10))  (-(1-delta) .* k + 1^(-10))];                              
  % xu
  out2  = [Inf(n,1) (ones(n,1)-1^(-10)) Inf(n,1)];

case 'f';
k = s(:,1);
a = s(:,2);
c = x(:,1);
l = x(:,2);
i = x(:,3);

  y     = exp(a) .* k .^ alpha .* l .^ (1-alpha);
  uc    = theta .* ((c .^ theta .* (1-l) .^ (1-theta)) .^ (1-tau)) ./ c;
  ikd   = i ./ k - delta;
  ikd1  = 1 + phi .* ikd;
  
  % f1
  out1(:,1) =  uc .* ikd1 - beta .* z;
  % f2
  out1(:,2) =  (1-theta) ./ theta .* l ./ (1-l) - (1-alpha) .* y ./ c;
  % f2
  out1(:,3) =  y - c - i - phi ./ 2 .* k .* ikd .^ 2;
 
  % f wrt s
  % f1_k
  out2(:,1,1) = -uc .* phi .* i ./ k .^ 2 ;
  % f1_a
  out2(:,1,2) = zeros(n,1);
  % f2_k
  out2(:,2,1) = y ./ k ./ c .* (alpha-1) .* alpha;
  % f2_a
  out2(:,2,2) = y ./ c .* (alpha-1);
  % f3_k
  out2(:,3,1) = y ./k .* alpha + phi ./ 2 .* ((i ./ k) .^ 2 - delta .^2 );
  % f3_a
  out2(:,3,2) = y;

  % f wrt x
  % f1_c
  out3(:,1,1) = uc ./ c .* (theta .* (1-tau) - 1) .* ikd1;
  % f1_l
  out3(:,1,2) = uc ./ (1-l) .* (1-theta) .* (tau-1) .* ikd1;
  % f1_i
  out3(:,1,3) = uc ./ k .* phi;
  % f2_c
  out3(:,2,1) = y ./ c.^2 .* (1-alpha);
  % f2_l
  out3(:,2,2) = (1-theta) ./ (1-l) .^ 2 ./ theta - y ./ c ./ l .* (1-alpha) .^ 2;
  % f2_i
  out3(:,2,3) = zeros(n,1);
  % f3_c
  out3(:,3,1) = -ones(n,1);
  % f3_l
  out3(:,3,2) = y ./ l .* (1-alpha);
  % f3_i
  out3(:,3,3) = -ikd1;

  % f wrt z
  % f1_z
  out4(:,1,1) = -beta .* ones(n,1);
  % f2_z
  out4(:,2,1) = zeros(n,1);
  % f3_z
  out4(:,3,1) = zeros(n,1);
    
case 'g';    
k = s(:,1);
a = s(:,2);
c = x(:,1);
l = x(:,2);
i = x(:,3);
    
  % g1
  out1(:,1)   = i + k .* (1-delta);
  % g2
  out1(:,2)   = rho .* a + e;

  % g wrt x
  % g1_c
  out2(:,1,1) = zeros(n,1);
  % g1_l
  out2(:,1,2) = zeros(n,1);
  % g1_i
  out2(:,1,3) = ones(n,1);
  % g2_c
  out2(:,2,1) = zeros(n,1);
  % g2_l
  out2(:,2,2) = zeros(n,1);
  % g2_i
  out2(:,2,3) = zeros(n,1);
  
  % g wrt s
  % g1_k
  out3(:,1,1) = ones(n,1) - delta;
  % g1_a
  out3(:,1,2) = zeros(n,1);
  % g2_k
  out3(:,2,1) = zeros(n,1);
  % g2_a
  out3(:,2,2) = rho .* ones(n,1);
  
case 'h';    
k = sn(:,1);
a = sn(:,2);
c = xn(:,1);
l = xn(:,2);
i = xn(:,3);
    
  y     = exp(a) .* k .^ alpha .* l .^ (1-alpha);
  uc    = theta .* ((c .^ theta .* (1-l) .^ (1-theta)) .^ (1-tau)) ./ c;
  ikd   = i ./ k - delta;
  ikd2  = phi ./ 2 .* ikd .* (ikd+2) + 1 - delta;
  
  % h1
  out1(:,1) = uc .* (y ./ k .* alpha + ikd2);

  % h wrt to x
  % h1_c
  out2(:,1,1) = zeros(n,1);
  % h1_l
  out2(:,1,2) = zeros(n,1);
  % h1_i
  out2(:,1,3) = zeros(n,1);

  % h wrt sn
  % h_k
  out3(:,1,1) = uc ./ k .* (y ./ k .* (alpha-1) .* alpha - i ./ k .* phi .* (ikd+1));
  % h_a
  out3(:,1,2) = uc .* y ./ k .* alpha;

  % h wrt to xn
  % h1_c
  out4(:,1,1) = out1(:,1) .* (theta .* (1-tau) - 1) ./ c;
  % h1_l
  out4(:,1,2) = uc ./ (1-l) .* (y ./ k ./ l .* alpha .* (1-alpha + l .* (alpha-2+theta+tau .* (1-theta)))-(theta-1) .* (tau-1) .* ikd2);
  % h1_i
  out4(:,1,3) = uc ./ k .* phi .* (ikd+1);

end
