% REMSIMUL  Monte Carlo simulation of discrete time controlled Markov process
% USAGE
%   [st xt] = remsimul(model,fspace,c,C,sss,xss,s0,nper,sol_lin)
% INPUTS
%   model   : name of model structure
%   fspace  : name of projection space structure
%   c       : policy function coefficients as returned from resolve
%   C       : policy matrix from linearization of the model
%   sss     : steady state of state
%   xss     : steady state of decision
%   s0      : k by ds vector of initial states
%   nper    : number of simulated time periods
%   sol_nonlin : nonlinear simulation
% OUTPUT
%   ssim    : simulated states
%   xsim    : simulated decision
%   ssimL   : simulated state with linear decision
%   xsimL   : simulated decision
%
% USES: DISCRAND

% 
% changed from remsolve by Viktor Winschel
%  added linear and nonlinear simulation on same shocks
% 
% Copyright (c) 1997-2000, Paul L. Fackler & Mario J. Miranda
% paul_fackler@ncsu.edu, miranda.4@osu.edu

function [ssim,xsim,ssimL,xsimL] = M_remsimul(model,fspace,c,C,sss,xss,s0,nper,sol_nonlin)

% DUMMY SHOCKS/WEIGHTS IF MODEL DETERMINISTIC  
  if isfield(model,'e'), e=model.e; else, e=0; end;
  if isfield(model,'w'), w=model.w; else, w=1; end;
  
  nrep = size(s0,1);
  ds   = size(s0,2);
  dx   = size(C,1);

  ssim = zeros(nrep,ds,nper+1);  
  xsim = zeros(nrep,dx,nper+1);
  ssimL = zeros(nrep,ds,nper+1);  
  xsimL = zeros(nrep,dx,nper+1);

  % set same seed for same random numbers
  rng(30);
    
  s0L = s0;
  for t=1:nper+1
    if sol_nonlin 
        xx  = funeval(c,fspace,s0);
    end
    xxL = repmat(xss,nrep,1) + (s0L - repmat(sss,nrep,1)) * C';

    if sol_nonlin 
        ssim(:,:,t) = s0;      
        xsim(:,:,t) = xx;
    end
    ssimL(:,:,t) = s0L;      
    xsimL(:,:,t) = xxL;
    ee = e(discrand(nrep,w),:);
    if sol_nonlin 
        s0 = feval(model.func,'g',s0,xx,[],ee,[],[],model.params{:});
    end
    s0L= feval(model.func,'g',s0L,xxL,[],ee,[],[],model.params{:});
  end
 
  if sol_nonlin 
    ssim = ssim(:,:,1:nper+1); 
    xsim = xsim(:,:,1:nper+1);
  end
  ssimL = ssimL(:,:,1:nper+1); 
  xsimL = xsimL(:,:,1:nper+1);