%% Code accompanying "Uncertainty Quantification and Global Sensitivity Analysis for Economic Models"
%% by Daniel Harenberg, Stefano Marelli, Bruno Sudret, and Viktor Winschel

% load the values
cd('results')
load('fig2_values.mat')
% values will be stored into "left_panel" and "right_panel"
VV(:,:,1) = left_panel;
VV(:,:,2) = right_panel;
% COLORS: just red and blue
COLS = [0 0 1; 1 0 0];
dx = 0.25;
VARNAMES = {'\beta','\tau', '\chi','\alpha','\delta', '\phi', '\rho','\sigma'};

YL = [-0.8 1.8; -0.8 1.8];

% loop over the panels
for ii = 1:2
    % Create a plot with the right size
    fname = sprintf('OAT%i',ii);
    uq_figure('Filename', fname, 'Name', fname)
    
    % Bar plot the values one beside each other
    xx = 1:8';
    uq_bar(xx-dx/2, VV(:,ii,2),dx, 'edgecolor','none', 'facecolor', COLS(2,:));
    hold on;
    uq_bar(xx+dx/2, VV(:,ii,1),dx, 'edgecolor','none', 'facecolor', COLS(1,:));
    
    %  Add the variable names
    % format the X-axis
    set(gca, 'XTick', 1:8, 'XTickLabel', VARNAMES)
    xlim([0 9]);
    % Y-axis
    ylim(YL(ii,:))
    
    % Meaningful legend
l = legend({'lower-baseline', 'baseline-upper'}, 'location', 'northwest','FontSize', 16);

    %saveas(gcf,fname,'png')
    print(fname,'-dpng')
end
cd('..')