%% Code accompanying "Uncertainty Quantification and Global Sensitivity Analysis for Economic Models"
%% by Daniel Harenberg, Stefano Marelli, Bruno Sudret, and Viktor Winschel
% Last modified: 29. March 2018

% clearvars;          % clear workspace
% uqlab;              % start uqlab
rng(100);           % reseed random number generator
clc;                % clear command window
EDFilename = 'UQLab_ED_EnrichLHS_50-10-500_allQoI.mat';
UQFilename = 'UQLab_FullModel_Input_and_Model_allQoI.mat';

global additional_QoIs % compute more than just the first two QoIs.
additional_QoIs = true;
%% 1 - Specify the probabilistic input
parNames{1} = '\beta';  % Discount factor                         
parNames{2} = '\tau';   % Elasticity of intertemporal substitution
parNames{3} = '\chi'; % Consumption and leisure substitution rate
parNames{4} = '\alpha'; % Technical substitution rate              
parNames{5} = '\delta'; % Depreciation rate                        
parNames{6} = '\phi';   % Capital adjustment cost parameter        
parNames{7} = '\rho';   % Autocorrelation of productivity          
parNames{8} = '\sigma'; % Standard deviation of productivity shock 

% parameter bounds          baseline values
parVals = [0.97,   0.99;        % beta  0.98
           1.00,   3.00;        % tau   2
           0.2,    0.4;         % theta 0.3
           0.2,    0.5;         % alpha 0.35
           0.01,   0.03;        % delta 0.02
           0.00,   1.0;         % phi   0.5
           0.92,   0.98;        % rho   0.95
           0.005,  0.009];      % sigma 0.007

    
for i=1:8                       % Independent marginals
    parNameTex = parNames{i};
    Input.Marginals(i).Name = parNameTex(2:end);
    Input.Marginals(i).Type = 'Uniform';
    Input.Marginals(i).Parameters = parVals(i,:);
end
myInput = uq_createInput(Input);    % Create input object

%% Specify the full model
Model.mFile = 'M_solve_RBC';
FullModel = uq_createModel(Model);                % create and add the model object to UQLab


%% Generate an experimental design by enriching an LHS
% Initial LHS, best of 5000 LHS in terms of maximin distance
XED = uq_getSample(50, 'LHS', 'iterations', 5000); 
for ii = 1:45
    XED = [XED; uq_LHSify(XED,10)];
end

%% Evaluate the full model on the generated exp design
YED = uq_evalModel(XED);

%% Save the results, as well as the UQLab session with input and full model
uq_saveSession(UQFilename);
save(EDFilename, 'XED', 'YED');